Package: dosedesignR
Type: Package
Title: Interactive Designing of Dose Finding Studies
Version: 0.3.0
Author: F. Mendolia, K. Walkamp, K. Roth, T. Schmelter, C. Neumann,  Z. Gao, A. Kaiser, S. Jeske, H. Kulmann
Maintainer: T. Schmelter <thomas.schmelter@bayer.com>
Description: Provides the user with an interactive application which can be used to facilitate the planning of dose finding studies by applying the theory of optimal experimental design. 
Depends: R (>= 3.6.0)
License: GPL-3
Encoding: UTF-8
Imports: utils, shiny, DT, latticeExtra, ggplot2, shinyjs, purrr,
        stringr, shinyWidgets, DoseFinding, shinyBS, readxl, knitr,
        kableExtra, Rsolnp
VignetteBuilder: knitr
NeedsCompilation: no
RoxygenNote: 7.2.3
Packaged: 2023-02-20 12:07:34 UTC; SGRPP
Repository: CRAN
Date/Publication: 2023-02-20 13:00:07 UTC
Built: R 4.2.0; ; 2023-07-11 01:16:20 UTC; unix
